[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]
[Qemu-devel] [PATCH v12 15/21] migration: Add block where to send/receiv
From: |
Juan Quintela |
Subject: |
[Qemu-devel] [PATCH v12 15/21] migration: Add block where to send/receive packets |
Date: |
Wed, 25 Apr 2018 13:27:17 +0200 |
Once there add tracepoints.
Signed-off-by: Juan Quintela <address@hidden>
---
migration/ram.c | 49 +++++++++++++++++++++++++++++++++++++-----
migration/trace-events | 2 ++
2 files changed, 46 insertions(+), 5 deletions(-)
diff --git a/migration/trace-events b/migration/trace-events
index e480eb050e..9eee048287 100644
--- a/migration/trace-events
+++ b/migration/trace-events
@@ -81,6 +81,8 @@ multifd_send_thread_start(uint8_t id) "%d"
multifd_send_thread_end(uint8_t id, uint32_t packets, uint32_t pages) "channel
%d packets %d pages %d"
multifd_recv_thread_start(uint8_t id) "%d"
multifd_recv_thread_end(uint8_t id, uint32_t packets, uint32_t pages) "channel
%d packets %d pages %d"
+multifd_send(uint8_t id, uint32_t seq, uint32_t used, uint32_t flags) "channel
%d seq number %d pages %d flags 0x%x"
+multifd_recv(uint8_t id, uint32_t seq, uint32_t used, uint32_t flags) "channel
%d seq number %d pages %d flags 0x%x"
# migration/migration.c
await_return_path_close_on_source_close(void) ""
diff --git a/migration/ram.c b/migration/ram.c
index 21b448c4ed..c4c185cc4c 100644
--- a/migration/ram.c
+++ b/migration/ram.c
@@ -455,6 +455,8 @@ typedef struct {
bool running;
/* should this thread finish */
bool quit;
+ /* thread has work to do */
+ int pending_job;
/* array of pages to sent */
MultiFDPages_t *pages;
/* packet allocated len */
@@ -489,6 +491,8 @@ typedef struct {
bool running;
/* should this thread finish */
bool quit;
+ /* thread has work to do */
+ bool pending_job;
/* array of pages to receive */
MultiFDPages_t *pages;
/* packet allocated len */
@@ -756,8 +760,28 @@ static void *multifd_send_thread(void *opaque)
while (true) {
qemu_sem_wait(&p->sem);
qemu_mutex_lock(&p->mutex);
- multifd_send_fill_packet(p);
- if (p->quit) {
+
+ if (p->pending_job) {
+ uint32_t used = p->pages->used;
+ uint32_t seq = p->seq;
+ uint32_t flags = p->flags;
+
+ multifd_send_fill_packet(p);
+ p->flags = 0;
+ p->num_packets++;
+ p->num_pages += used;
+ p->pages->used = 0;
+ qemu_mutex_unlock(&p->mutex);
+
+ trace_multifd_send(p->id, seq, used, flags);
+
+ /* ToDo: send packet here */
+
+ qemu_mutex_lock(&p->mutex);
+ p->pending_job--;
+ qemu_mutex_unlock(&p->mutex);
+ continue;
+ } else if (p->quit) {
qemu_mutex_unlock(&p->mutex);
break;
}
@@ -823,6 +847,7 @@ int multifd_save_setup(void)
qemu_mutex_init(&p->mutex);
qemu_sem_init(&p->sem, 0);
p->quit = false;
+ p->pending_job = 0;
p->id = i;
multifd_pages_init(&p->pages, page_count);
p->packet_len = sizeof(MultiFDPacket_t)
@@ -910,14 +935,27 @@ static void *multifd_recv_thread(void *opaque)
while (true) {
qemu_sem_wait(&p->sem);
qemu_mutex_lock(&p->mutex);
- if (false) {
- /* ToDo: Packet reception goes here */
+ if (p->pending_job) {
+ uint32_t used;
+ uint32_t flags;
+ qemu_mutex_unlock(&p->mutex);
+ /* ToDo: recv packet here */
+
+ qemu_mutex_lock(&p->mutex);
ret = multifd_recv_unfill_packet(p, &local_err);
- qemu_mutex_unlock(&p->mutex);
if (ret) {
+ qemu_mutex_unlock(&p->mutex);
break;
}
+
+ used = p->pages->used;
+ flags = p->flags;
+ trace_multifd_recv(p->id, p->seq, used, flags);
+ p->pending_job = false;
+ p->num_packets++;
+ p->num_pages += used;
+ qemu_mutex_unlock(&p->mutex);
} else if (p->quit) {
qemu_mutex_unlock(&p->mutex);
break;
@@ -960,6 +998,7 @@ int multifd_load_setup(void)
qemu_mutex_init(&p->mutex);
qemu_sem_init(&p->sem, 0);
p->quit = false;
+ p->pending_job = false;
p->id = i;
multifd_pages_init(&p->pages, page_count);
p->packet_len = sizeof(MultiFDPacket_t)
--
2.17.0
- [Qemu-devel] [PATCH v12 06/21] migration: Create multifd channels, (continued)
- [Qemu-devel] [PATCH v12 06/21] migration: Create multifd channels, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 07/21] migration: Delay start of migration main routines, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 08/21] migration: Transmit initial package through the multifd channels, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 09/21] migration: Define MultifdRecvParams sooner, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 10/21] migration: Create multipage support, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 11/21] migration: Create multifd packet, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 12/21] migration: Add multifd traces for start/end thread, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 13/21] migration: Calculate transferred ram correctly, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 14/21] migration: Multifd channels always wait on the sem, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 15/21] migration: Add block where to send/receive packets,
Juan Quintela <=
- [Qemu-devel] [PATCH v12 18/21] migration: Start sending messages, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 17/21] migration: Create ram_multifd_page, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 16/21] migration: Synchronize multifd threads with main thread, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 20/21] migration: Remove not needed semaphore and quit, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 19/21] migration: Wait for blocking IO, Juan Quintela, 2018/04/25
- [Qemu-devel] [PATCH v12 21/21] migration: Stop sending whole pages through main channel, Juan Quintela, 2018/04/25
- Re: [Qemu-devel] [PATCH v12 00/21] Multifd, Juan Quintela, 2018/04/25
- Re: [Qemu-devel] [PATCH v12 00/21] Multifd, Peter Xu, 2018/04/26